;***********************************************************************
;*              Lastftwindow modification    .  .2003
;***********************************************************************
;*              Device and microprogramm identification
;***********************************************************************

.equ    _day=          29               ; Date of the programming
.equ    _month=        12
.equ    _year=          1
.equ    pdate=(_year*16+_month)*32+_day
                                        ; Device code
.equ    _author=        0
.equ    _group=         6               ; Device group
.equ    _device=        1               ; Device serial number
.equ    iddev=(_author*16+_group)*256+_device

;***********************************************************************
;*              Microcontroller input/output definitions
;***********************************************************************
                                        ; Port D
.equ    RX=             PD0             ; UART Input
.equ    TX=             PD1             ; UART Output
.equ    RXI=            PD2             ; Line monitor input (INT0)
.equ    EXT=            PD3             ; Synchro clocks input (INT1)
.equ    T0=             PD4             ; A: Counter T0 input/MSBit of pre-counter
.equ    T1=             PD5             ; B: Counter T1 input/MSBit of pre-counter
.equ    DE=             PD6             ; RS485 transmission enable output
.equ	MR=		PD7             ; Pre-counters reset output
.equ    DDMask=         0b11000010
.equ    DMask=          0b10111101

                                        ; Port B
.equ    CSA=            PB0             ; Select output for AD8400_A loading
.equ    CSB=            PB1             ; Select output for AD8400_B loading
.equ    SS=           	PB2             ; SS
.equ    STROB=          PB3             ; Output synchro clocks (OC2)/SPI (MOSI)/ SDI for AD8400
.equ    MISO=           PB4             ; SPI (MISO)
.equ    SCK=            PB5             ; Clock output for AD8400 loading/SPI
.equ    BDMask=         0b00101111
.equ	BMask=          0b00000111
                                        ; Port C
.equ    DB0=            PC0             ; Bit 0 of pre-counterA input
.equ    DB1=            PC1             ; Bit 1 of pre-counterA input
.equ    DB2=            PC2             ; Bit 2 of pre-counterA input
.equ    DA0=            PC3             ; Bit 0 of pre-counterB input
.equ    DA1=            PC4             ; Bit 1 of pre-counterB input
.equ    DA2=		PC5             ; Bit 2 of pre-counterB input
.equ    CDMask=         0b00000000
.equ    CMask=          0b00111111

;***********************************************************************
;*              Working register definitions
;***********************************************************************
.def    SWITCH=         R0      ; RS485 timer switch
.def    TOP=            R1      ; RS485 timer window
.def    LINE_TIMER=     R2      ; RS485 timer counter
.def    CYCLES=         R3      ; Exposure cycles counter
.def    EXPL=           R4      ; Exposure regester low
.def    EXPH=           R5      ; Exposure regester high
.def    CNT3=           R6      ; Most significant counter0 (expand 8bit Counter0 to 16)
.def    SREG_COPY=      R7      ; SREG copy register for all interrupt subrouts
.def    AL=             R8    	; CRC, Command decoding
.def    AH=             R9	; CRC, Command decoding
.def    ARGUMENT=       R10     ; Command decoding
.def    DATAS=          R11     ; PortD copy
.def    DATA0=          R12
.def    DATA1=          R13
.def    DATA2=          R14
.def    BYTE_CNT=      	R15     ; Receiver & Transmitter byte count

.def    STATUS=         R16     ; STATUS
.def    LINE_CSR=       R17     ; LINE STATUS register
.def    POINTER=        R18     ; Shift relativly first data byte
.def    TREG=           R19     ; Temoral low register for interrupt
.def    TEMP=           R20     ; Temporal register for backgrond
.def    TEMP_CNT=      	R21     ; Temporal register for backgrond
.def    RECEIVE=       	R22     ; Temporal receiver register
.def    COMMAND=        R22     ; Temporal command register
.def    TRANSMIT=       R23     ; Temporal transmitter register

.def    NUMBL=          R24     ; Exposition serie lenght register low
.def    NUMBH=          R25     ; Exposition serie lenght register high

.equ    DIVISOR=	7	; for CRC
.equ    MAXADRESS=    	0x1F
.equ    MODIFICATOR=    4	; command with 2 byte argument

;***********************************************************************
;*              Status registers flags
;***********************************************************************
                                        ; STATUS  register
.equ    ACTIVE=         0              	; Active mode on
.equ    INDUCED=        1              	; Inductive mode on
.equ    SHORTED=        2              	; Short format of data transmission on
.equ    EXTERNAL=       3               ; Integration using external clock
.equ    TEST=           4               ; Decremental test on
.equ    BLOCK_READY=    5               ; Data block is ready for transmission
.equ    DATA_READY=     6               ; New count ready (Microexpos. finished)
.equ    MEASUREMENT=    7              	; Integration in progress

                                        ; LINE register
.equ    RECEIVING=      7              	; Packet receiving in progress
.equ    RECV_READY=     6              	; Packet receiving is complete
.equ    LINE_BUSY=      5              	; Line is busy
.equ    FAILURE=        4               ; Transmission failure is detected
.equ    TRANS_START=    3               ; Data is ready for transmission
.equ    DATA_SENDING=   2               ; Data block is transmitted
.equ    INDUCE_PASS=    1               ; Inductor packet was passed
.equ    INDUCTOR=       0               ; This is inductor packet

;***********************************************************************
;*              Protocol Signals
;***********************************************************************
.equ    ACK= 0x87                       ; Successful receiving of data
.equ    NAK= 0x96                       ; Damaged packet was received
.equ    NOD= 0xA5                       ; Required data are not ready
.equ    ACN= 0xB4                       ; Such a command does not exist
.equ    ACY= 0xC3                       ; Command is successfully received and executed
.equ    ACW= 0xD2                       ; Command can not be executed right now
.equ    SINC=0xE1                       ; Synchronization signal
.equ    DNG= 0xF0                       ; Danger signal

;***********************************************************************
;*              Exchange constants
;***********************************************************************
.equ    COUNTER=        0x01            ; module address
.equ    EXCHANGE_RATE=  460             ; Default Exchange rate in KBod
.equ    WINDOW=         500  		; Reply window duration (mks 550 max)
.equ    DELAY=          25           	; Transmittance switch delay (mks 10 min)
.equ    NLINE=          4               ; Number of line parameters
.equ    MAXRETRY=       2               ; Max retry number (1 max)
.equ    freq=           14746           ; Oscillator frequency (cycles/ms)

.equ    fwindow=        (WINDOW*freq/1000/14/8);       +1
.equ    fswitch=        (DELAY*freq/1000/14/8);        +1
.equ    fbaudrate=   	(freq/EXCHANGE_RATE)/16-1 ;  Max baudrate=-1 => U2X = 1

;***********************************************************************
;*              Functional constants
;***********************************************************************
.equ    PAUSE=          12              ; Pause between strob pulse (mks, 65 max)
.equ    fstrob_pause=   (PAUSE*freq/1000/8)
.equ    data_size=      64              ; Size of double buffer for count

